;**********************************************************************
;*                                                                    *
;*     Rotina para determinar o ponto de fuga melhor ajustado         *
;*     a um conjunto de rectas com a mesma direco perspectivadas    *
;**********************************************************************

;**********************************************************************
;*                    Portugal - Lisboa, Fevereiro, 2008              *
;*                                                                    *
;*     Escrito por Lus Miguel Cotrim Mateus - lmmateus@fa.utl.pt     *
;*                                                                    *
;*     Faculdade de Arquitectura da Universidade Tcnica de Lisboa    *
;*                                                                    *
;**********************************************************************



;1 funo auxiliar: 	conduz por um ponto 1P1 uma recta perpendicular a uma recta definida por dois pontos 2P1 e 2P2 definido mais um ponto 1P1n
(defun per_2p (1P1 2P1 2P2)
  (setq 1P1x (car 1P1))
  (setq 1P1y (cadr 1P1))
  (setq 2P1x (car 2P1))
  (setq 2P1y (cadr 2P1))
  (setq 2P2x (car 2P2))
  (setq 2P2y (cadr 2P2))
  	(setq 2dx (- 2P1x 2P2x))
  	(setq 2dy (- 2P1y 2P2y))
  	; coordenadas de um ponto de uma recta 1- passante por 1P1 perpendicular a 2P1.2P2
  	(setq 1P1-x (+ 1P1x 2dy))
  	(setq 1P1-y (- 1P1y 2dx))
  	(setq 1P1n (list 1P1-x 1P1-y))
)


;2 funo auxiliar:	atribui um peso (confiana)  interseco entre duas rectas
(defun weight (1P1 1P2 2P1 2P2)
  (setq 1P1x (car 1P1))
  (setq 1P1y (cadr 1P1))
  (setq 1P2x (car 1P2))
  (setq 1P2y (cadr 1P2))
  
  (setq 2P1x (car 2P1))
  (setq 2P1y (cadr 2P1))
  (setq 2P2x (car 2P2))
  (setq 2P2y (cadr 2P2))
  
  	;chamada  1 funo 	coordenadas de um ponto 1P1- de uma recta passante por 1P1 perpendicular a 2P1.2P2
	(per_2p 1P1 2P1 2P2)
  	(setq 1P1- 1P1n)

  	;chamada  1 funo 	coordenadas de um ponto de uma recta 2- passante por 1P2 perpendicular a 2P1.2P2
	(per_2p 1P2 2P1 2P2)
  	(setq 1P2- 1P1n)
  
  	; pontos de interseco entre as duas rectas e a recta 2P1.2P2
  	(setq int_1-2 (inters 1P1 1P1- 2P1 2P2 nil))
  	(setq int_2-2 (inters 1P2 1P2- 2P1 2P2 nil))

  	; clculo do sin do ngulo entre as duas rectas
  	(setq hipotenusa (distance 1P1 1P2))
  	(setq cateto (distance int_1-2 int_2-2))
  	(setq seno (sqrt (- 1 (* (/ cateto hipotenusa) (/ cateto hipotenusa)))))

	; peso da interseco
  	(setq comp_2P12P2 (distance 2P1 2P2))
  	(setq peso (* hipotenusa comp_2P12P2 seno))
)


;3 funo auxiliar:	determina as coordenadas dos pontos de interseco entre rectas e respectivo peso
(defun int_2r (1P1 1P2 2P1 2P2) 
  (setq Pt_int (inters 1P1 1P2 2P1 2P2 nil))
	(setq X_Pt_int (car Pt_int))
  	(setq Y_Pt_int (cadr Pt_int))

  	; chamada  2 funo
  	(weight 1P1 1P2 2P1 2P2)
  	(setq W peso)
)


;3 funo auxiliar'':	determina as coordenadas dos pontos de interseco entre rectas e respectivo peso
(defun int_2r- (1P1 1P2 2P1 2P2) 
  (setq Pt_int (inters 1P1 1P2 2P1 2P2 nil))
	(setq X_Pt_int (car Pt_int))
  	(setq Y_Pt_int (cadr Pt_int))
)


;4 funo auxiliar:	agrupa numa lista li a os pares de pontos que definem as rectas de uma famlia
(defun sr (a)

  (setq li (list ))
  
  (setq n (sslength a))
  
  (setq k (- n 1))

  (while k

  (setq nent (ssname a k))
  (setq lent (entget nent))

  (setq lPn1 (assoc 10 lent))
  (setq lPn2 (assoc 11 lent))

  (setq x1 (cadr lPn1))
  (setq y1 (caddr lPn1))

  (setq x2 (cadr lPn2))
  (setq y2 (caddr lPn2))

  (setq recta (list (+ k 1) (list x1 y1) (list x2 y2) ))
    
  (setq l (list recta))
  (setq lt (append l li))
  (setq li lt)

    (setq k (- k 1))
    
  (if (= k -1) (setq k nil) )
    
   )
)


; 5 funo auxiliar
; determina as interseces entre todas rectas da lista b
; agrupa numa lista as coordenadas X ou Y dos pontos de interseco
; agrupa numa lista o peso de cada ponto


(defun sl (b)

  (setq li_x (list ))
  (setq li_y (list ))
  (setq li_w (list ))
  
  (setq nn (vl-list-length b))
  
  (setq kk 1)

(while kk

  	(setq 1_recta (assoc kk b))
	(setq 1Pt1 (cadr 1_recta))
  	(setq 1Pt2 (caddr 1_recta))
  
  (setq kkk (+ kk 1))

(while kkk

  	(setq 2_recta (assoc kkk b))
  	(setq 2Pt1 (cadr 2_recta))
  	(setq 2Pt2 (caddr 2_recta))

 	; chamada  3 funo
  	(int_2r 1Pt1 1Pt2 2Pt1 2Pt2)
  
	(setq li_x_aux (append (list (* X_Pt_int W)) li_x))
  	(setq li_x li_x_aux)

	(setq li_y_aux (append (list (* Y_Pt_int W)) li_y))
  	(setq li_y li_y_aux)

  	(setq li_W_aux (append (list W) li_w))
	(setq li_W li_W_aux)

   (setq kkk (+ kkk 1))
  
   (if (= kkk (+ nn 1)) (setq kkk nil))
  )
  
(setq kk (+ kk 1))

  	
	(if (= kk nn) (setq kk nil))
	)
)


; 6 funo auxiliar 
; calcula as coordenadas do ponto de fuga
; note-se que a mdia aritmtica neste caso cumpre o requisito dos mnimos quadrados
(defun coor_PF (li_x li_y li_w)

(setq soma_li_x (apply '+ li_x))
(setq soma_li_y (apply '+ li_y))
(setq soma_li_w (apply '+ li_w))

	(setq F_x (/ soma_li_x soma_li_w))
 	(setq F_y (/ soma_li_y soma_li_w))

  	(setq PF (list F_x F_y))
)


; 7 funo auxiliar:	cria o ponto de fuga atravs de uma lista
(defun ponto_fuga (C_x C_y)
  (setq lista_0_p (cons 0 "POINT"))
  (setq lista_10_p (list 10 C_x C_y))
  (setq lista_p (list lista_0_p lista_10_p))
  (entmake lista_p)
)


; 8 funo auxiliar:	redesenha linhas a partir do ponto de fuga determinado
(defun redr_lines (PF l_zed)

  (setq var1 1)

  (while var1

	(setq mid_list (assoc var1 l_zed))
    	(setq mid_pt_1 (cadr mid_list))
    	(setq mid_pt_2 (caddr mid_list))
    	(setq mid_pt (list (/ (+ (car mid_pt_1) (car mid_pt_2)) 2) (/ (+ (cadr mid_pt_1) (cadr mid_pt_2)) 2)))

    	; chamada  1 funo
	(per_2p mid_pt_1 PF mid_pt)
    	(setq 1Pt_l_red- 1P1n)

    	; chamada  1 funo
    	(per_2p mid_pt_2 PF mid_pt)
    	(setq 2Pt_l_red- 1P1n)

    	; determinao dos pontos da linha redesenhada
    	; chamada  3 funo''
    	(int_2r- mid_pt_1 1Pt_l_red- PF mid_pt)
    	(setq 1Pt_l_red Pt_int)
    
	; chamada  3 funo''
    	(int_2r- mid_pt_2 2Pt_l_red- PF mid_pt)
    	(setq 2Pt_l_red Pt_int)

        ; redesenha a recta atravs de uma lista
	(setq lista_0 (cons 0 "LINE"))
    	(setq lista_10 (list 10 (car 1Pt_l_red) (cadr 1Pt_l_red)))
    	(setq lista_11 (list 11 (car 2Pt_l_red) (cadr 2Pt_l_red)))
    	(setq lista (list lista_0 lista_10 lista_11))
    	(entmake lista)

    (setq var1 (+ var1 1))

    (if (= var1 (+ n 1)) (setq var1 nil))
   )
)



; 9 funo auxiliar:	seleco de rectas paralelas a zz
(defun zz ()
      	(print "Selecione uma famlia de rectas paralelas a ZZ")
	(setq a (ssget))

	; cria uma lista que agrupa as rectas paralelas a zz
  	; chamada  4 funo
  	(sr a)
  	(setq l_zz li)
  
  	; determina os pontos de interseco entre as rectas paralelas a zz
	; agrupando-os numa lista
	; chamada  5 funo
  	(sl l_zz)
  )



; 10 funo auxiliar:	 seleco de rectas horizontais da fachada esquerda
(defun esq ()
       	(print "Selecione uma famlia de rectas horizontais da fachada  esquerda")
  	(setq a (ssget))

	; cria uma lista que agrupa as rectas paralelas do lado esquerdo
  	; chamada  4 funo
  	(sr a)
	(setq l_esq li)
  
  	
  	; determina os pontos de interseco entre as rectas horizontais da fachada esq
	; agrupando-os numa lista
  	; chamada  5 funo
  	(sl l_esq)
  ) 

; 11 funo auxiliar:	seleco de rectas horizontais da fachada direita
(defun dta ()
       	(print "Selecione uma famlia de rectas horizontais da fachada  direita")
  	(setq a (ssget))

  	; cria uma lista que agrupa as rectas horizontais da fachada direita
  	; chamada  4 funo
  	(sr a)
  	(setq l_dta li)
  
	; determina os pontos de interseco entre as rectas horizontais da fachada dta
	; agrupando-os numa lista
  	; chamada  5 funo
  	(sl l_dta)
  ) 


; 12 funo auxiliar:	seleco de rectas paralelas de uma direco
(defun vv ()
      	(print "Selecione uma famlia de rectas paralelas")
	(setq a (ssget))

	; cria uma lista que agrupa as rectas paralelas a zz
  	; chamada  4 funo
  	(sr a)
  	(setq l_vv li)
  
  	; determina os pontos de interseco entre as rectas paralelas a zz
	; agrupando-os numa lista
	; chamada  5 funo
  	(sl l_vv)
  )


; 13 funo auxiliar:	pede a seleco de dois pontos e d o ponto mdio desenhando-o atravs de uma lista
(defun pt_med ()
  	(setq med_pt1 (getpoint "\n1 extremo da diagonal da foto:"))
	(setq med_pt2 (getpoint "\n2 extremo da diagonal da foto:"))

  	(setq med_x (/ (+ (car med_pt1) (car med_pt2)) 2))
  	(setq med_y (/ (+ (cadr med_pt1) (cadr med_pt2)) 2))

  	;desenha o ponto PP
  	;chamada  7 funo
	(ponto_fuga med_x med_y)
)

; 14 funo auxiliar:	desenha uma recta dadas as coordenadas de dois pontos
(defun drecta (x y w z)
	(setq lis_0 (cons 0 "LINE"))
    	(setq lis_10 (list 10 x y))
    	(setq lis_11 (list 11 w z))
    	(setq lis (list lis_0 lis_10 lis_11))
    	(entmake lis)
)


; 15 funo auxiliar: desenha uma circunferncia dado o centro e o raio
(defun circ (x_cent y_cent radi)
  	(setq lisc_0 (cons 0 "CIRCLE"))
  	(setq lisc_100 (cons 100 "AcDbCircle"))
  	(setq lisc_10 (list 10 x_cent y_cent))
  	(setq lisc_40 (cons 40 radi))
  	(setq lisc (list lisc_0 lisc_100 lisc_10 lisc_40))
  	(entmake lisc)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;   No grupo de funes principais 1, 1a, 1b, 1c determina-se um ponto de         ;
;   fuga, redesenham-se linhas que devero ter a direco seleccionada e          ;
;   desenham-se linhas a partir do ponto de fuga determinado.                     ;
;   Pode ainda determinar-se o ponto principal.                                   ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;FUNO PRINCIPAL 1:	 determina o ponto de fuga de uma direco
(defun c:spf ()

  
; chamada  9 funo
  	(vv)

  	; chamada  6 funo
  	(coor_PF li_x li_y li_w)
  	(setq PF_v PF)

  	;  7 funo
  	(ponto_fuga (car PF_v) (cadr PF_v))  
)


;FUNO PRINCIPAL 1a:	 determina o ponto de fuga de uma direco e redesenha as linhas em funo desse ponto 
(defun c:spf_r ()

  
; chamada  9 funo
  	(vv)

  	; chamada  6 funo
  	(coor_PF li_x li_y li_w)
  	(setq PF_v PF)

  	;  7 funo
  	(ponto_fuga (car PF_v) (cadr PF_v))

	; redesenha as linhas da lista l_vv em funo do novo ponto de fuga
  	; chamada  8 funo
  	(redr_lines PF_v l_vv)
)

;FUNO PRINCIPAL 1b: 	desenha linhas a partir do ponto de fuga PF_v
(defun c:sp_lv ()
  	(command "line" PF_v )
)


;FUNO PRINCIPAL 1c: 	desenha o Ponto Principal PP
(defun c:pp_ ()

	;calcula as coordenadas do Ponto Principal PP
       	;chamada  13 funo
       	(pt_med)

	(setq PPx med_x)
  	(setq PPy med_y)

  	;desenha o ponto PP
  	;chamada  7 funo
	(ponto_fuga PPx PPy)
)






