;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                                                             ;
;	Programa para simular a tomada de fotos sobre um objecto (apenas definido por linhas rectas)          ;
;                                                                                                             ;
; 	Escrito por Lus Mateus, Janeiro de 2010, Lisboa      -     Faculdade de Arquitectura da UTL          ;
;                                                                                                             ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


; FUNO PRINCIPAL 01 - DEFINIO DA CMARA
; definio da cmara em funo da distncia principal (dP), da largura do sensor (W), da altura do sensor (H),
; e de um factor de escala (K) para tornar visvel a cmara no desenho

(defun c:camara ()
  (setq dP (getreal "\distncia principal em mm: "))
  (setq W (getreal "\largura do sensor em mm: "))
  (setq H (getreal "\altura do sensor em mm: "))
  (setq K (getreal "\factor de escala (sugesto K=0.1): "))

  (setq xAi (* K (/ W 2)))
  (setq yAi (* K (/ H 2)))
  
  (setq xBi (* K (/ W 2)))
  (setq yBi (* K (- (/ H 2))))
  
  (setq xCi (* K (- (/ W 2))))
  (setq yCi (* K (- (/ H 2))))

  (setq xDi (* K (- (/ W 2))))
  (setq yDi (* K (/ H 2)))
  
  (setq zi (* K (- dP)))
					; controlo
					;(setq ptAi (list xAi yAi zi))
					;(setq ptBi (list xBi yBi zi))
					;(setq ptCi (list xCi yCi zi))
					;(setq ptDi (list xDi yDi zi))
  					;(setq ptOi (list 0 0 0))
  					;(command "osmode" "0")
					;(command "line" ptAi ptBi ptCi ptDi ptOi ptAi ptDi "")
  					;(command "osmode" "3")
  					; fim de controlo
  (princ)
)


; FUNO PRINCIPAL 02 - SELECO DO OBJECTO           

(defun c:objecto ()

	(print "Seleccione o objecto (apenas segmentos de recta)...")
  
	(setq apt (ssget))

  	(setq npt (sslength apt))

  (princ)
)

;FUNO PRINCIPAL 03 - ORIENTAO DA CMARA E TOMADA DE IMAGEM
; definio da orientao da cmara atravs do centro de projeces (O), do alvo (T), e de um ngulo de rotao (teta) em torno do eixo da objectiva

	;	1 - definio da cmara na origem

(defun c:foto ()
  (setq Est (getpoint "\nEstao: "))
  (setq Alvo (getpoint "\nAlvo: "))
  (setq teta (getreal "\angulo de rotao em torno do eixo (0 a 180): "))
  
  (setq tet (/ (* pi teta) 180))
  (setq AlvoH (list (car Alvo) (cadr ALvo) (caddr Est)))
  (setq AlvoX (list (car Alvo) (cadr Est) (caddr Est)))

  
	;	2 - definio da cmara no espao
  
		; definio do ngulo gama, o ngulo gama corresponde  inclinao do eixo da mquina e  funo da
  		; diferena de cotas entre os pontos de Estao e Alvo
		(setq cs_gama (/ (distance Est AlvoH) (distance Est Alvo)))
  		(setq sn_gama (/ (- (caddr Alvo) (caddr AlvoH)) (distance Est Alvo)))
			(setq pt1 (list 0 0))
  			(setq pt2 (list cs_gama sn_gama))
		(setq gama (angle pt1 pt2))
  		; definio do ngulo pii, o ngulo pii  o ngulo que a projeco horizontal do eixo da mquina faz com o eixo x mundo
		(setq cs_pii (/ (- (car AlvoX) (car Est)) (distance Est AlvoH)))
  		(setq sn_pii (/ (- (cadr AlvoH) (cadr AlvoX)) (distance Est AlvoH)))
			(setq pt3 (list cs_pii sn_pii))
			
		(setq pii (angle pt1 pt3))


		; colocao do plano da foto em posio no espao atravs de uma rotao (90+gama) em torno de x, uma rotao de -(90-Pii) em torno de z,
		; e de uma translao (XO, YO, ZO)
		;	2.1 - rotao (90+gama) em torno de x
  
		;		ptA

		(rot tet xAi yAi)
     			     (setq xAir ar)
     			     (setq yAir br)
     			     (setq zAir zi)
  
  		(rot (+ (/ pi 2) gama) yAir zAir)
  			     (setq yAirx ar)
     			     (setq zAirx br)
  			     (setq xAirx xAir)

  		(rot (- (- (/ pi 2) pii)) xAirx yAirx)
  			     (setq xAirxz ar)
  			     (setq yAirxz br)
  			     (setq zAirxz zAirx)

  		(transla (car Est) (cadr Est) (caddr Est) xAirxz yAirxz zAirxz)
  			     (setq xAirxzt ct)
  			     (setq yAirxzt dt)
  			     (setq zAirxzt et)

		(setq ptAirxzt (list xAirxzt yAirxzt zAirxzt))
 
  		;		ptB

		(rot tet xBi yBi)
     			     (setq xBir ar)
     			     (setq yBir br)
     			     (setq zBir zi)

  		(rot (+ (/ pi 2) gama) yBir zBir)
  			     (setq yBirx ar)
     			     (setq zBirx br)
  			     (setq xBirx xBir)

  		(rot (- (- (/ pi 2) pii)) xBirx yBirx)
  			     (setq xBirxz ar)
  			     (setq yBirxz br)
  			     (setq zBirxz zBirx)

  		(transla (car Est) (cadr Est) (caddr Est) xBirxz yBirxz zBirxz)
  			     (setq xBirxzt ct)
  			     (setq yBirxzt dt)
  			     (setq zBirxzt et)

		(setq ptBirxzt (list xBirxzt yBirxzt zBirxzt))

  		;		ptC

		(rot tet xCi yCi)
     			     (setq xCir ar)
     			     (setq yCir br)
     			     (setq zCir zi)
  
   		(rot (+ (/ pi 2) gama) yCir zCir)
  			     (setq yCirx ar)
     			     (setq zCirx br)
  			     (setq xCirx xCir)

  		(rot (- (- (/ pi 2) pii)) xCirx yCirx)
  			     (setq xCirxz ar)
  			     (setq yCirxz br)
  			     (setq zCirxz zCirx)

  		(transla (car Est) (cadr Est) (caddr Est) xCirxz yCirxz zCirxz)
  			     (setq xCirxzt ct)
  			     (setq yCirxzt dt)
  			     (setq zCirxzt et)

		(setq ptCirxzt (list xCirxzt yCirxzt zCirxzt))

  		;		ptD

		(rot tet xDi yDi)
     			     (setq xDir ar)
     			     (setq yDir br)
     			     (setq zDir zi)
  
    		(rot (+ (/ pi 2) gama) yDir zDir)
  			     (setq yDirx ar)
     			     (setq zDirx br)
  			     (setq xDirx xDir)

  		(rot (- (- (/ pi 2) pii)) xDirx yDirx)
  			     (setq xDirxz ar)
  			     (setq yDirxz br)
  			     (setq zDirxz zDirx)

  		(transla (car Est) (cadr Est) (caddr Est) xDirxz yDirxz zDirxz)
  			     (setq xDirxzt ct)
  			     (setq yDirxzt dt)
  			     (setq zDirxzt et)

		(setq ptDirxzt (list xDirxzt yDirxzt zDirxzt))

  		;		ptP
  		(setq ptP (list (/ (+ xDirxzt xBirxzt) 2) (/ (+ yDirxzt yBirxzt) 2) (/ (+ zDirxzt zBirxzt) 2)))

  		;		fidS
		(setq fidS (list (/ (+ xDirxzt xAirxzt) 2) (/ (+ yDirxzt yAirxzt) 2) (/ (+ zDirxzt zAirxzt) 2)))
  
		;		desenho do plano de imagem
					
  					(command "osmode" "0")
					(command "line" ptAirxzt ptBirxzt ptCirxzt ptDirxzt ptAirxzt "")
  					(command "line" Est ptP fidS "")
  					(command "osmode" "3")
  
  		;		chamada  funo que define a imagem

(c:snapshot)
  
)
  
; FUNO PRINCIPAL 04 - SNAPSHOT - funo que define a imagem       
(defun c:snapshot ()


(command "osmode" "0")
  
  
  (setq kk (- npt 1))

  (while kk

  (setq nent (ssname apt kk))
  (setq lent (entget nent))

  (setq lPn1 (assoc 10 lent))
  (setq lPn2 (assoc 11 lent))

	;                    1 endpoint
    
(setq xn1 (cadr lPn1))
(setq yn1 (caddr lPn1))
(setq zn1 (cadddr lPn1))


		; translao (-XO, -YO, -Z0)

	 (transla (- (car Est)) (- (cadr Est)) (- (caddr Est)) xn1 yn1 zn1)
	   	(setq xn1t ct)
  		(setq yn1t dt)
  		(setq zn1t et)

    		; rotao (90-Pi) em torno de z

  	 (rot (- (/ pi 2) pii) xn1t yn1t)
  		(setq xn1tz ar)
  		(setq yn1tz br)
  		(setq zn1tz zn1t)

   		; rotao de -(90+gama) em torno de x
	 
    	 (rot (- (+ (/ pi 2) gama)) yn1tz zn1tz)
  		(setq yn1tzx ar)
     		(setq zn1tzx br)
  		(setq xn1tzx xn1tz)

    		; rotao de -teta em torno de z

	 (rot (- tet) xn1tzx yn1tzx)
  		(setq xn1tzxe ar)
  		(setq yn1tzxe br)
  		(setq zn1tzxe zn1tzx)

    		; projeco cnica na origem
 
	(conica xn1tzxe yn1tzxe zn1tzxe dP K)
	 	(setq xp1 fp)
	 	(setq yp1 gp)
	 	(setq zp1 hp)
    						; controlo
						;(setq listaa (list 10 xp1 yp1 zp1))
    
		; rotao de +teta em torno de z
    
	(rot tet xp1 yp1)
  		(setq xp1e ar)
  		(setq yp1e br)
  		(setq zp1e zp1)

						; controlo
    						;(setq lista1 (list 10 xp1e yp1e zp1e))
    
		; rotao de (90+gama) em torno de x aplicada  perspectiva
    
    	 (rot (+ (/ pi 2) gama) yp1e zp1e)
  		(setq yp1ex ar)
     		(setq zp1ex br)
  		(setq xp1ex xp1e)
    
    						; controlo
    						;(setq lista1 (list 10 xp1ex yp1ex zp1ex))
    
		; rotao -(90-Pi) em torno de z aplicada  perspectiva rodada em torno de x
    
  	 (rot (-(- (/ pi 2) pii)) xp1ex yp1ex)
 		(setq xp1exz ar)
		(setq yp1exz br)
		(setq zp1exz zp1ex)

    						; controlo
    						;(setq lista1 (list 10 xp1exz yp1exz zp1exz))
    
    		; translao (XO, YO, Z0) aplicada  perspectiva rodada em torno de z
    
    	  (transla (car Est) (cadr Est) (caddr Est) xp1exz yp1exz zp1exz)
	   	(setq xp1exzt ct)
 		(setq yp1exzt dt)
		(setq zp1exzt et)
    
		; contruo do ponto imagem orientado
    
	  (setq lista1 (list 10 xp1exzt yp1exzt zp1exzt))
  
	;                     2 endpoint
	 
(setq xn2 (cadr lPn2))
(setq yn2 (caddr lPn2))
(setq zn2 (cadddr lPn2))


		; translao (-XO, -YO, -Z0)

	 (transla (- (car Est)) (- (cadr Est)) (- (caddr Est)) xn2 yn2 zn2)
	   	(setq xn2t ct)
  		(setq yn2t dt)
  		(setq zn2t et)

    		; rotao (90-Pi) em torno de z

  	 (rot (- (/ pi 2) pii) xn2t yn2t)
  		(setq xn2tz ar)
  		(setq yn2tz br)
  		(setq zn2tz zn2t)

   		; rotao de -(90+gama) em torno de x
	 
    	 (rot (-(+ (/ pi 2) gama)) yn2tz zn2tz)
  		(setq yn2tzx ar)
     		(setq zn2tzx br)
  		(setq xn2tzx xn2tz)

    		; rotao de -teta em torno de z

	 (rot (- tet) xn2tzx yn2tzx)
  		(setq xn2tzxe ar)
  		(setq yn2tzxe br)
  		(setq zn2tzxe zn2tzx)

    		; projeco cnica na origem
 
	(conica xn2tzxe yn2tzxe zn2tzxe dP K)
	 	(setq xp2 fp)
	 	(setq yp2 gp)
	 	(setq zp2 hp)

    					; controlo
					;(setq listab (list 11 xp2 yp2 zp2))
    
		; rotao de +teta em torno de z
    
	(rot tet xp2 yp2)
  		(setq xp2e ar)
  		(setq yp2e br)
  		(setq zp2e zp2)

    					; controlo
    					;(setq lista2 (list 11 xp2e yp2e zp2e))
    
		; rotao de (90+gama) em torno de x aplicada  perspectiva
    
    	 (rot (+ (/ pi 2) gama) yp2e zp2e)
  		(setq yp2ex ar)
     		(setq zp2ex br)
  		(setq xp2ex xp2e)

    					; controlo
    					;(setq lista2 (list 11 xp2ex yp2ex zp2ex))
    
		; rotao -(90-Pi) em torno de z aplicada  perspectiva rodada em torno de x
    
  	 (rot (-(- (/ pi 2) pii)) xp2ex yp2ex)
 		(setq xp2exz ar)
		(setq yp2exz br)
		(setq zp2exz zp2ex)

    					; controlo
    					;(setq lista2 (list 11 xp2exz yp2exz zp2exz))
    
    		; translao (XO, YO, Z0) aplicada  perspectiva rodada em torno de z
    
 	  (transla (car Est) (cadr Est) (caddr Est) xp2exz yp2exz zp2exz)
	   	(setq xp2exzt ct)
		(setq yp2exzt dt)
		(setq zp2exzt et)


		; contruo do ponto imagem orientado
	 
		(setq lista2 (list 11 xp2exzt yp2exzt zp2exzt))
    
	; 		linha recta
(setq lista0 (cons 0 "LINE"))
    
					; controlo
					;(setq listac (list lista0 listaa listab))
    					;(entmake listac)
    
(setq lista (list lista0 lista1 lista2))

(entmake lista)

	  
  (setq kk (- kk 1))
    
  (if (= kk -1) (setq kk nil) )		
   )

	(setq lista nil)

  	(princ)
  
  (command "osmode" "3")
  
  )





  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; FUNES AUXILIARES

  
; rotao 
(defun rot (ang a b)
  (setq ar (- (* a (cos ang)) (* b (sin ang))))
  (setq br (+ (* a (sin ang)) (* b (cos ang))))
  )

; translao
(defun transla (deltaX deltaY deltaZ c d e)
  (setq ct (+ c deltaX))
  (setq dt (+ d deltaY))
  (setq et (+ e deltaZ))
  )

; projeco cnica na origem
(defun conica (f g h dP K)

(setq fp (* K (* (/ (- dP) h) f)))
(setq gp (* K (* (/ (- dP) h) g)))
(setq hp (* K (- dP)))
  )
      
  

  










  
