;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                                                                       ;
;	Programa para simular a tomada de fotos esfricas sobre um objecto (apenas definido por linhas rectas)          ;
;                                                                                                                       ;
; 	Escrito por Lus Mateus, Fevereiro de 2010, Lisboa      -     Faculdade de Arquitectura da UTL                  ;
;                                                                                                                       ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


; FUNO PRINCIPAL 01 - DEFINIO DA CMARA
; definio da cmara em funo da distncia principal (dP) 
; e de um factor de escala (K) para tornar visvel a cmara no desenho

(defun c:cam_esf ()
  (setq raio (getreal "\distncia principal em mm: "))
  (setq K_esf (getreal "\factor de escala (sugesto K=0.1): "))

	(command "osmode" "0")
	; circulo do plano neutro colocado na origem
  		(setq centro (list 0 0))
  		(setq Kraio (* K_esf raio))
  		(setq Kraio_p (/ (* 2 pi Kraio) 4))
		(command "circle" centro Kraio "")

  	; circulo do plano da imagem colocado na origem
  		(setq centroi (list 0 0 (- Kraio)))
  		(setq vert (list 0 Kraio_p (- Kraio)))
  		(command "circle" centroi Kraio_p "")

  	; desenho da distncia principal e eixo y da imagem colocados na origem
  		(command  "line" centro centroi vert "")
  		(command "osmode" "3")
  				
  (princ)
)


; FUNO PRINCIPAL 02 - SELECO DO OBJECTO           

(defun c:objecto ()

	(print "Seleccione o objecto (apenas segmentos de recta)...")
  
	(setq apt (ssget))

  	(setq npt (sslength apt))

  (princ)
)

;FUNO PRINCIPAL 03 - ORIENTAO DA CMARA E CHAMADA  TOMADA DE IMAGEM
	
	;	1 - Definio do centro de projeces (O), e do alvo (T)
(defun c:foto_esf ()
  (setq Est_esf (getpoint "\nEstao: "))
  (setq Alvo_esf (getpoint "\nAlvo: "))

  (setq Alvo_esfH (list (car Alvo_esf) (cadr ALvo_esf) (caddr Est_esf)))
  (setq Alvo_esfX (list (car Alvo_esf) (cadr Est_esf) (caddr Est_esf)))

  (setq pt1 (list 0 0))

  
  
	;	2.	definio da cmara no espao
(command "osmode" "0")

  		;2.1.	 definio do vector normal ao plano neutro da camara - PU, este vector  paralelo ao eixo da cmara
		(setq dist_E_A (distance Est_esf Alvo_esf))
  
  		(setq D_X (-  (car Est_esf) (car Alvo_esf) ))
  		(setq D_Y (-  (cadr Est_esf) (cadr Alvo_esf) ))
		(setq D_Z (-  (caddr Est_esf) (caddr Alvo_esf) ))

  		
  		(setq UX (/ D_X dist_E_A) )
		(setq UY (/ D_Y dist_E_A) )
  		(setq UZ (/ D_Z dist_E_A) )

  		(setq PU (list UX UY UZ))
  
		; 2.2.		definio do circulo do plano neutro e do circulo do plano da imagem
  		;
  		; 			para definir uma circulo como lista
  		; 			 preciso expressar as coordenadas do seu centro no
  		;			sistema de coordenadas com origem comum ao WCS
  		; 			e eixos definidos pelo vector PU
		; 			Para isso  preciso determinar 2 ngulos: o angulo pii_esf e o ngulo gama_esf
  		
  
		; 2.2.1. 	definio do ngulo gama_esf
  		;
  		;			o ngulo gama_esf corresponde  inclinao do eixo da mquina e  funo da
  		; 			diferena de cotas entre os pontos de Estao e Alvo
		(setq cs_gama_esf (/ (distance Est_esf Alvo_esfH) (distance Est_esf Alvo_esf)))
  		(setq sn_gama_esf (/ (- (caddr Alvo_esfH) (caddr Alvo_esf) ) (distance Est_esf Alvo_esf)))
			
  			(setq pt2_esf (list cs_gama_esf sn_gama_esf))
		(setq gama_esf (angle pt1 pt2_esf))
  
  		; 2.2.2.	definio do ngulo pii_esf
  		;
  		; 			o ngulo pii_esf  o ngulo que a projeco horizontal do eixo da mquina faz com o eixo x mundo
		(if
		  (= (distance Est_esf Alvo_esfH) 0)
		  (setq pii_esf (/ pi 2))
		  (progn
			(setq cs_pii_esf (/ (- (car Alvo_esfX) (car Est_esf)) (distance Est_esf Alvo_esfH)))
  			(setq sn_pii_esf (/ (- (cadr Alvo_esfH) (cadr Alvo_esfX)) (distance Est_esf Alvo_esfH)))
				(setq pt3_esf (list cs_pii_esf sn_pii_esf))	
				(setq pii_esf (angle pt1 pt3_esf))
		    )
		  )

  		; 2.2.3.	determinao do centro do circulo do plano neutro
  		;
  		; 			rotao do referencial WCS para ficar paralelo ao referencial definido pelo vector PU, deste
  		;			modo podem extrair-se as coordenadas do centro do circulo no novo referencial.
  		;			Para o efeito deve efectuar-se
		; 			uma rotao do referencial por -(90-Pii_esf) em torno de z, e por (90-gama_esf) em torno de x rodado <=>
  		; 			<=> rotao do centro por (90-Pii_esf) em torno de z e por -(90-gama_esf) em torno de x rodado

  			; rotao em torno de z
		(rot (- (/ pi 2) Pii_esf)  (car Est_esf) (cadr Est_esf) )
  			(setq x_Est_esf_z ar)
     			(setq y_Est_esf_z br)
     			(setq z_Est_esf_z (caddr Est_esf))

  
			; rotao em torno de x rodado
  		(rot (- (- (/ pi 2) gama_esf)) y_Est_esf_z z_Est_esf_z )
			(setq x_Est_esf_zx  x_Est_esf_z)
     			(setq y_Est_esf_zx ar)
     			(setq z_Est_esf_zx br)

  			; centro do crculo do plano neutro

  		(setq Est_esf_zx (list x_Est_esf_zx y_Est_esf_zx z_Est_esf_zx))

  			; construo da lista do circulo do plano neutro e raio igual  distncia principal
  		(setq listac0 (cons 0 "CIRCLE"))
  		(setq listac8 (cons 8 "foto_e"))
  		(setq listac10 (cons 10 Est_esf_zx))
  		(setq listac40 (cons 40 Kraio))
  		(setq listac210 (cons 210 PU))
  		(setq listac (list listac0 listac8 listac40 listac10 listac210))
  		(entmake listac)

  		; 2.2.4.	determinao do centro do circulo do plano da imagem
  		;
		; 			a cota do centro deste crculo difere da cota do centro do circulo do plano
  		;			neutro pelo raio deste (diferena expressa no referencial PU)
  
  			; centro do crculo do plano da imagem
  		(setq z_p (-  z_Est_esf_zx kraio))

  		(setq Pprinc (list x_Est_esf_zx y_Est_esf_zx z_p))


  			; construo da lista do circulo da imagem
  		(setq listai10 (cons 10 Pprinc))
  		(setq listai40 (cons 40 Kraio_p))
   		(setq listai (list listac0 listac8 listai40 listai10 listac210))
  		(entmake listai) 		

  		; 2.2.5. 	determinao do ponto principal e vertical da imagem (eixo y da imagem)
  
			; as coordenadas so expressas em funo do ponto estao (Est_esf), do vector PU, e da distncia principal escalada (Kraio)
  		(setq XPP (-  (car Est_esf) (* Kraio UX) ))
  		(setq YPP (-  (cadr Est_esf) (* Kraio UY) ))
  		(setq ZPP (-  (caddr Est_esf) (* Kraio UZ) ))

  		(setq PP (list XPP YPP ZPP))

  			; definio da vertical relativa da imagem no espao
			; uma rotao por (90-gama) em torno de x e -(90-Pii) em torno de z

  			; rotao em torno de x
  		(rot (- (/ pi 2) gama_esf) (cadr vert) (caddr vert) )
			(setq XPVx (car vert))
     			(setq YPVx ar)
     			(setq ZPVx br)

			; rotao em torno de z
		(rot (- (- (/ pi 2) Pii_esf)) XPVx YPVx)
  			(setq XPVxz ar)
     			(setq YPVxz br)
     			(setq ZPVxz ZPVx)
  
			; translao
  		(transla (car Est_esf) (cadr Est_esf) (caddr Est_esf) XPVxz YPVxz ZPVxz)
			(setq XPVxzt ct)
  			(setq YPVxzt dt)
  			(setq ZPVxzt et)

  			; lista das coordenadas do ponto "mais alto" do plano da imagem
  		(setq PV (list XPVxzt YPVxzt ZPVxzt))
		
			; desenho da distncia principal e do eixo y da imagem
		(command "line" Est_esf PP PV "")
  
 (command "osmode" "3")

  		; 2.2.6.	Chamada a funo que vai definir as imagens
  
(c:snap_esf)
  
)




; FUNO PRINCIPAL 04- SNAP_ESF - funo que define a imagem       
(defun c:snap_esf ()


(command "osmode" "0")
  
  
  (setq kk (- npt 1))

  (while kk

  (setq nent (ssname apt kk))
  (setq lent (entget nent))

  (setq Pn1 (assoc 10 lent))
  (setq Pn2 (assoc 11 lent))

	; 1.	Translaes e rotaes para fazer coincidir o referencial com o referencial mundo
	;                    1 endpoint

		; translao por - as coordenadas de Est_est
	(transla (- (car Est_esf)) (- (cadr Est_esf)) (- (caddr Est_esf)) (cadr Pn1) (caddr Pn1) (cadddr Pn1))
    	(setq XPn1t ct)
    	(setq YPn1t dt)
    	(setq ZPn1t et)
    	
    		; rotao (90-Pii_esf) em torno de z
	(rot (- (/ pi 2) Pii_esf) XPn1t YPn1t)
    	(setq XPn1tz ar)
    	(setq YPn1tz br)
    	(setq ZPn1tz ZPn1t)

    		; rotao de -(90-gama_esf) em torno de x
    	(rot (- (- (/ pi 2) gama_esf)) YPn1tz ZPn1tz)
	(setq xn1 XPn1tz)
    	(setq yn1 ar)
    	(setq zn1 br)
    
    
   	;                     2 endpoint

		; translao por - as coordenadas de Est_est
	(transla (- (car Est_esf)) (- (cadr Est_esf)) (- (caddr Est_esf)) (cadr Pn2) (caddr Pn2) (cadddr Pn2))
    	(setq XPn2t ct)
    	(setq YPn2t dt)
    	(setq ZPn2t et)
    	
    		; rotao (90-Pii_esf) em torno de z
	(rot (- (/ pi 2) Pii_esf) XPn2t YPn2t)
    	(setq XPn2tz ar)
    	(setq YPn2tz br)
    	(setq ZPn2tz ZPn2t)

    		; rotao de -(90-gama_esf) em torno de x
    	(rot (- (- (/ pi 2) gama_esf)) YPn2tz ZPn2tz)
	(setq xn2 XPn2tz)
    	(setq yn2 ar)
    	(setq zn2 br)

	; 2.	Projeces na origem + planificaes na origem
	;			midpoint

    	(setq xm (/ (+ xn1 xn2) 2))
    	(setq ym (/ (+ yn1 yn2) 2))
    	(setq zm (/ (+ zn1 zn2) 2))


	; 			projeco do 1 endpoint
    
	(con_esf xn1 yn1 zn1 raio K_esf)
	 	(setq xp1 fpe)
	 	(setq yp1 gpe)
	 	(setq zp1 hpe)

    						

	; 			projeco do 2 endpoint

	(con_esf xn2 yn2 zn2 raio K_esf)
	 	(setq xp2 fpe)
	 	(setq yp2 gpe)
	 	(setq zp2 hpe)
    
						
	; 			projeco do midpoint

	(con_esf xm ym zm raio K_esf)
	 	(setq xmp fpe)
	 	(setq ymp gpe)
	 	(setq zmp hpe)
    
	;			planificao do 1 endpoint

	(planif K_esf raio xp1 yp1 zp1)
    		(setq xp1p lp)
    		(setq yp1p mp)
    		(setq zp1p (* K_esf (- raio)))

    		(setq Pt1p (list xp1p yp1p zp1p))

 	;			planificao do 2 endpoint

	(planif K_esf raio xp2 yp2 zp2)
    		(setq xp2p lp)
    		(setq yp2p mp)
    		(setq zp2p (* K_esf (- raio)))
    
    		(setq Pt2p (list xp2p yp2p zp2p))

  	;			planificao do midpoint

	(planif K_esf raio xmp ymp zmp)
    		(setq xmpp lp)
    		(setq ympp mp)
    		(setq zmpp (* K_esf (- raio)))

    		(setq Ptmp (list xmpp ympp zmpp))

    	; 3.	Definio de critrio para definir se a imagem de um segmento  uma recta ou um arco
    	; 	+ Definio das linhas e dos arcos na origem e no espao
    	;
	; 			arco ou linha? Se DIF_1>0 ento  arco, Se DIF_1=0 ento  linha

    		; 		distncias entre os pontos dos arcos
    		(setq Pt1p_Pt2p (distance Pt1p Pt2p))
    		(setq Pt1p_Ptmp (distance Pt1p Ptmp))
    		(setq Pt2p_Ptmp (distance Pt2p Ptmp))

    	(setq DIF_l (- (+ Pt1p_Ptmp Pt2p_Ptmp) Pt1p_Pt2p))

    		(setq listal0 (cons 0 "LINE"))
    		(setq listaa0 (cons 0 "ARC"))
    		(setq listao8 (cons 8 "foto_o"))
    								
		; 		definio das linhas e dos arcos

    			(progn
			  (if
			  ; condio 1
			    (< DIF_l (* Kraio 0.000001))
			  ; se condio 1 = T
			    (progn
			        ; recta na origem
			      		; lista da recta na origem
				(setq listal10 (cons 10 Pt1p))
				(setq listal11 (cons 11 Pt2p))
				(setq listal (list  listal0 listao8 listal10 listal11))
				(entmake listal)
			        ; recta no espao
			      		; 1 endpoint
  					; rotao em torno de x
  					(rot (- (/ pi 2) gama_esf) yp1p zp1p )  
						(setq xp1px xp1p)
     						(setq yp1px ar)
     						(setq zp1px br)

					; rotao em torno de z
					(rot (- (- (/ pi 2) Pii_esf)) xp1px yp1px)
  						(setq xp1pxz ar)
     						(setq yp1pxz br)
     						(setq zp1pxz zp1px)
  
					; translao
  					(transla (car Est_esf) (cadr Est_esf) (caddr Est_esf) xp1pxz yp1pxz zp1pxz)
						(setq xp1pxzt ct)
  						(setq yp1pxzt dt)
  						(setq zp1pxzt et)

  						(setq Pt1pxzt (list xp1pxzt yp1pxzt zp1pxzt))

			      		; 2 endpoint
  					; rotao em torno de x
  					(rot (- (/ pi 2) gama_esf) yp2p zp2p )  
						(setq xp2px xp2p)
     						(setq yp2px ar)
     						(setq zp2px br)

					; rotao em torno de z
					(rot (- (- (/ pi 2) Pii_esf)) xp2px yp2px)
  						(setq xp2pxz ar)
     						(setq yp2pxz br)
     						(setq zp2pxz zp2px)
  
					; translao
  					(transla (car Est_esf) (cadr Est_esf) (caddr Est_esf) xp2pxz yp2pxz zp2pxz)
						(setq xp2pxzt ct)
  						(setq yp2pxzt dt)
  						(setq zp2pxzt et)

  						(setq Pt2pxzt (list xp2pxzt yp2pxzt zp2pxzt))
					; lista da recta no espao
					(setq listale10 (cons 10 Pt1pxzt))
					(setq listale11 (cons 11 Pt2pxzt))
					(setq listale (list  listal0 listac8 listale10 listale11))
					(entmake listale)
			     )
			    
			  ; else (se condio 1 = F)
			    (progn

				; ponto mdio Pt1p_Ptmp mais normal
    
				(setq xm1 (/ (+ (car Pt1p) (car Ptmp)) 2))
				(setq ym1 (/ (+ (cadr Pt1p) (cadr Ptmp)) 2))
				(setq zm1 (/ (+ (caddr Pt1p) (caddr Ptmp)) 2))

    					(setq Ptm1 (list xm1 ym1 zm1))

    					(setq Ptm1_p (list (+ xm1 (- yp1p ympp) ) (+ ym1 (- xmpp xp1p) ) zm1))

    

    				; ponto mdio Pt2p_Ptmp mais normal
				(setq xm2 (/ (+ (car Pt2p) (car Ptmp)) 2))
				(setq ym2 (/ (+ (cadr Pt2p) (cadr Ptmp)) 2))
				(setq zm2 (/ (+ (caddr Pt2p) (caddr Ptmp)) 2))

    					(setq Ptm2 (list xm2 ym2 zm2))

    					(setq Ptm2_p (list (+ xm2 (- yp2p ympp) ) (+ ym2 (- xmpp xp2p) ) zm2))

    				; centro e raio do arco -> interseco das duas normais

    				(setq ACen (inters Ptm1 Ptm1_p Ptm2 Ptm2_p nil))

    				(setq RArc (distance ACen Ptmp))

    				; ngulos inicial e final do arco

    				(setq APt1p (angle ACen Pt1p))
			      
    				(setq APt2p (angle ACen Pt2p))
			      
			        ; listas do centro e raio do arco

				(setq listaa10 (cons 10 ACen))

    				(setq listaa40 (cons 40 RArc))

					; lista do centro do arco no espao
					; o centro do arco  expresso como uma tranlao das coordenadas encontradas em relao ao WCS pelos
			      		; valores das coordenadas do centro de projeces expressas noo referencial definido pelo vector PU
					(transla x_Est_esf_zx y_Est_esf_zx z_Est_esf_zx (car ACen) (cadr ACen) (caddr ACen))
			      		(setq XACen_e ct)
			      		(setq YACen_e dt)
			      		(setq ZACen_e et)

			      		(setq ACen_e (list XACen_e YACen_e ZACen_e))

			      	(setq listaa10_e (cons 10 ACen_e))
			      		
				; determinao do ngulo inicial e do ngulo final do arco 

			        (if
				; condio 2 dentro da alternativa ELSE da 1 condio if
					(or
					  (and (> xn1 0) (> yn1 0) (> xn2 0) (or (< yn2 0) (= yn2 0)))
					  (and (> xn1 0) (or (< yn1 0) (= yn1 0)) (> xn2 0) (> yn2 0))
					  (and (> xn1 0) (< yn1 0) (> xn2 0) (or (> yn2 0) (= yn2 0)))
					  (and (> xn1 0) (= yn1 0) (> xn2 0) (< yn2 0))
					  )

				  	; se condio 2 = T
				  	(progn
				    
				  		(if 

				    			(< APt1p Apt2p)
					  
							(and (setq APtip APt2p) (setq APtfp APt1p))
				
							(and (setq APtip APt1p) (setq APtfp APt2p))
					 	)
				     	)

				  	; else (se condio 2 = F)
				  	(progn

				 		(if 
				    			(< APt1p Apt2p)
				     
							(and (setq APtip APt1p) (setq APtfp APt2p))

							(and (setq APtip APt2p) (setq APtfp APt1p))
				     		)
				    	)

				)
					
				    	; listas para a construo do arco na origem
    				(setq listaa50 (cons 50 APtip))

    				(setq listaa51 (cons 51 APtfp))

    				(setq listaa (list listaa0 listao8 listaa10 listaa40 listaa50 listaa51))
		
    				(entmake listaa)

			      		; lista para a construo do arco no espao

				(setq listaa_e (list listaa0 listac8 listaa10_e listaa40 listaa50 listaa51 listac210))

			      	(entmake listaa_e)
			      )
			    )
			  )
						
	  
  (setq kk (- kk 1))
    
  (if (= kk -1) (setq kk nil) )		
   )

  	(princ)
  
  (command "osmode" "3")
  
  )

    

    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; FUNES AUXILIARES

  
; rotao 
(defun rot (ang a b)
  (setq ar (- (* a (cos ang)) (* b (sin ang))))
  (setq br (+ (* a (sin ang)) (* b (cos ang))))
  )

; translao
(defun transla (deltaX deltaY deltaZ c d e)
  (setq ct (+ c deltaX))
  (setq dt (+ d deltaY))
  (setq et (+ e deltaZ))
  )

; projeco cnica sobre a esfera na origem
(defun con_esf (fe ge he raio K_esf)

(setq fpe (* K_esf (* (/ raio (sqrt (+ (* fe fe) (* ge ge) (* he he)))) fe)))
(setq gpe (* K_esf (* (/ raio (sqrt (+ (* fe fe) (* ge ge) (* he he)))) ge)))
(setq hpe (* K_esf (* (/ raio (sqrt (+ (* fe fe) (* ge ge) (* he he)))) he)))
  )

; planificao

(defun planif (K_esf raio l m n)
  (setq ptaux (list 0 0))
  (setq ptaux_1 (list (- n) (sqrt (+ (* l l) (* m m)))))
  (setq angulo (angle ptaux ptaux_1))
  (setq perim (* K_esf (* raio angulo)))

  (if
    (= perim 0)
    (and
        (setq lp 0)
	(setq mp 0)
	)
    (and
  	(setq lp  (* l (/ perim (sqrt (+ (* l l) (* m m))))))
  	(setq mp  (* m (/ perim (sqrt (+ (* l l) (* m m))))))
	)
     )
  )