;**********************************************************************
;*                                                                    *
;*     Rotina para  resolver transformaes projectivas sobre         *
;*     desenhos produzidos exclusivamente com linhas rectas           *
;*                                                                    *
;**********************************************************************

;**********************************************************************
;*                    Portugal - Lisboa, Dezembro, 2007               *
;*                                                                    *
;*     Escrito por Lus Miguel Cotrim Mateus - lmmateus@fa.utl.pt     *
;*                                                                    *
;*     Faculdade de Arquitectura da Universidade Tcnica de Lisboa    *
;*                                                                    *
;**********************************************************************

;rotao

(defun rot ( cs sn x y)
  (setq xr (- (* x cs) (* y sn)))
  (setq yr (+ (* x sn) (* y cs)))
  )

;translaco

(defun trs ( tx ty x y)
  (setq xt (+ x tx))
  (setq yt (+ y ty))  
)

;definio de parmetros projectivos auxiliares e0, e1, e2, f0, f1, f2, g1, g2

(defun parametros ( Pt1 Pt2 Pt3 Pt4 Pt11 Pt22 Pt33 Pt44)

(setq x1 (car Pt1))
(setq y1 (cadr Pt1))

(setq x2 (car Pt2))
(setq y2 (cadr Pt2))

(setq x3 (car Pt3))
(setq y3 (cadr Pt3))

(setq x4 (car Pt4))
(setq y4 (cadr Pt4))

(setq w1 (car Pt11))
(setq z1 (cadr Pt11))

(setq w2 (car Pt22))
(setq z2 (cadr Pt22))

(setq w3 (car Pt33))
(setq z3 (cadr Pt33))

(setq w4 (car Pt44))
(setq z4 (cadr Pt44))
  
;                     translao dos pontos do 1 quadriltero com o vector [Pt1 O]

(trs (- x1) (- y1) x1 y1)
  (setq x1tr xt)
  (setq y1tr yt)

(trs (- x1) (- y1) x2 y2)
  (setq x2t xt)
  (setq y2t yt)

(trs (- x1) (- y1) x3 y3)
  (setq x3t xt)
  (setq y3t yt)

(trs (- x1) (- y1) x4 y4)
  (setq x4t xt)
  (setq y4t yt)

;                     translaco dos pontos do 2 quadriltero com o vector [Pt44 0]
  
(trs (- w4) (- z4) w1 z1)
  (setq w1t xt)
  (setq z1t yt)

(trs (- w4) (- z4) w2 z2)
  (setq w2t xt)
  (setq z2t yt)

(trs (- w4) (- z4) w3 z3)
  (setq w3t xt)
  (setq z3t yt)

(trs (- w4) (- z4) w4 z4)
  (setq w4tr xt)
  (setq z4tr yt)
 
;                     rotao dos pontos do 1 quadriltero translaccionados anteriormente,
;                     com rotao em torno da origem com ngulo definido pelo vector [O P2t]

(setq dist1 (sqrt (+ (* x2t x2t) (* y2t y2t))))

  (setq cosa1 (/ x2t dist1))
  (setq sena1 (/ (- y2t) dist1))

(rot cosa1 sena1 x2t y2t)
  (setq x2tr xr)
  (setq y2tr yr)

(rot cosa1 sena1 x3t y3t)
  (setq x3tr xr)
  (setq y3tr yr) 
  
(rot cosa1 sena1 x4t y4t)
  (setq x4tr xr)
  (setq y4tr yr)

;                     rotao dos pontos do 2 quadriltero translaccionados anteriormente,
;                     com rotao em torno da origem com ngulo definido pelo vector [O P44t]

(setq dist2 (sqrt (+ (* w3t w3t) (* z3t z3t))))

  (setq cosa2 (/ w3t dist2))
  (setq sena2 (/ (- z3t) dist2))

(rot cosa2 sena2 w1t z1t)
  (setq w1tr xr)
  (setq z1tr yr)

(rot cosa2 sena2 w2t z2t)
  (setq w2tr xr)
  (setq z2tr yr)

(rot cosa2 sena2 w3t z3t)
  (setq w3tr xr)
  (setq z3tr yr)

;                    e0, e1, e2, f0, f1, f2, g1, g2

(setq g1 w1tr)
  
(setq g2 z1tr)

(setq e2 (/
	   (-
	     (* z1tr y3tr)
	     (* z1tr y4tr)
	     )
	   (-
	     (* x3tr y4tr)
	     (* y3tr x4tr)
	     )
	   )
      )

(setq f2 (/
	   (-
	     (* z1tr x4tr)
	     (* z1tr x3tr)
	     )
	   (-
	     (* x3tr y4tr)
	     (* y3tr x4tr)
	     )
	   )
      )
  

(setq e1 (+
	   (* e2 (/ w2tr z2tr))
	   (* w2tr
	      (/
		(- z1tr z2tr)
		(* x2tr z2tr)
		)
	      )
	   (-
	     (/
	       (- w1tr w2tr)
	       x2tr
	       )
	     )
	   )
      )


(setq f1 (-
	   (/
	     (+ (* e1 x4tr) w1tr)
	     y4tr
	     )
	   )
      )

  

(setq e0 (+
	   (/ e2 z2tr)
	   (/
	     (- z1tr z2tr)
	     (* x2tr z2tr)
	     )
	   )
      )


(setq f0 (/
	   (+
	     (* e1 x3tr)
	     (* f1 y3tr)
	     w1tr
	     (- w3tr)
	     (-
	       (* e0 x3tr w3tr)
	       )
	     )
	   (* y3tr w3tr)
	   )
      )
)

;transformao projectiva auxiliar

(defun proj ( e0 e1 e2 f0 f1 f2 g1 g2 xntr yntr)
  (setq Xutr (/ (+ (* e1 xntr) (* f1 yntr) g1) (+ (* e0 xntr) (* f0 yntr) 1)))
  (setq Yutr (/ (+ (* e2 xntr) (* f2 yntr) g2) (+ (* e0 xntr) (* f0 yntr) 1)))
)

;definio dos pontos de controlo quando se seleccionam linhas
(defun pts_ctrl ( la lb)

(setq n_la (car la))
(setq lla (entget n_la))
(setq lP1la (assoc 10 lla))
(setq lP2la (assoc 11 lla))
(setq e1la (list (cadr lP1la) (caddr lP1la)))
(setq e2la (list (cadr lP2la) (caddr lP2la)))      

(setq n_lb (car lb))
(setq llb (entget n_lb))
(setq lP1lb (assoc 10 llb))
(setq lP2lb (assoc 11 llb))
(setq e1lb (list (cadr lP1lb) (caddr lP1lb)))
(setq e2lb (list (cadr lP2lb) (caddr lP2lb)))

(setq Pi_a.b (inters e1la e2la e1lb e2lb nil))
)

;tranformao projectiva

(defun c:transf ()

  (command "osmode" "0")

(setq -sena2 (- sena2))

(print "Seleccione apenas segmentos de recta...")

  (setq apt (ssget))
  (setq npt (sslength apt))
  
  (setq kk (- npt 1))

  (while kk

  (setq nent (ssname apt kk))
  (setq lent (entget nent))

  (setq lPn1 (assoc 10 lent))
  (setq lPn2 (assoc 11 lent))

;                    1 endpoint
    
(setq xn1 (cadr lPn1))
(setq yn1 (caddr lPn1))

(trs (- x1) (- y1) xn1 yn1)
  (setq xn1t xt)
  (setq yn1t yt)

(rot cosa1 sena1 xn1t yn1t)
  (setq xn1tr xr)
  (setq yn1tr yr)
    
(proj e0 e1 e2 f0 f1 f2 g1 g2 xn1tr yn1tr)

  (setq xu1tr xutr)
  (setq yu1tr yutr)

(rot cosa2 -sena2 xu1tr yu1tr)
  (setq xu1t xr)
  (setq yu1t yr) 

(trs w4 z4 xu1t yu1t)
  (setq xu1 xt)
  (setq yu1 yt)
  
(setq lista1 (list 10 xu1 yu1))
  
;                     2 endpoint
    
(setq xn2 (cadr lPn2))
(setq yn2 (caddr lPn2))

(trs (- x1) (- y1) xn2 yn2)
  (setq xn2t xt)
  (setq yn2t yt)

(rot cosa1 sena1 xn2t yn2t)
  (setq xn2tr xr)
  (setq yn2tr yr)
 
(proj e0 e1 e2 f0 f1 f2 g1 g2 xn2tr yn2tr)

  (setq xu2tr xutr)
  (setq yu2tr yutr)
    
(rot cosa2 -sena2 xu2tr yu2tr)
  (setq xu2t xr)
  (setq yu2t yr) 

(trs w4 z4 xu2t yu2t)
  (setq xu2 xt)
  (setq yu2 yt)

(setq lista2 (list 11 xu2 yu2))   

(setq lista0 (cons 0 "LINE"))

(setq lista (list lista0 lista1 lista2))

(entmake lista)
    
  (setq kk (- kk 1))
    
  (if (= kk -1) (setq kk nil) )		
   )

  (command "osmode" "3")
  
)

;escolha de 4 pontos + 4 pontos e transformao

(defun c:4p4p1 ()
  (setq sc-de 1)
  (setq sc-para 0)
(setq Pt1 (getpoint "\nPonto de Controlo 1:"))
  (setq Pt1 (trans Pt1 sc-de sc-para))
(setq Pt2 (getpoint "\nPonto de Controlo 2:"))
  (setq Pt2 (trans Pt2 sc-de sc-para))
(setq Pt3 (getpoint "\nPonto de Controlo 3:"))
  (setq Pt3 (trans Pt3 sc-de sc-para))
(setq Pt4 (getpoint "\nPonto de Controlo 4:"))
  (setq Pt4 (trans Pt4 sc-de sc-para))
(setq Pt11 (getpoint "\nPonto correspondente a 1:"))
  (setq Pt11 (trans Pt11 sc-de sc-para))
(setq Pt22 (getpoint "\nPonto correspondente a 2:"))
  (setq Pt22 (trans Pt22 sc-de sc-para))
(setq Pt33 (getpoint "\nPonto correspondente a 3:"))
  (setq Pt33 (trans Pt33 sc-de sc-para))
(setq Pt44 (getpoint "\nPonto correspondente a 4:"))
  (setq Pt44 (trans Pt44 sc-de sc-para))

(command "osmode" "0")

(parametros Pt1 Pt2 Pt3 Pt4 Pt11 Pt22 Pt33 Pt44)
(c:transf)

(print)
)

;escolha de 1 ponto + 1 ponto (4xs) e transformao

(defun c:1p1p4 ()
  (setq sc-de 1)
  (setq sc-para 0)
(setq Pt1 (getpoint "\nPonto de Controlo 1:"))
  (setq Pt1 (trans Pt1 sc-de sc-para))
(setq Pt11 (getpoint "\nPonto correspondente a 1:"))
  (setq Pt11 (trans Pt11 sc-de sc-para))
(setq Pt2 (getpoint "\nPonto de Controlo 2:"))
  (setq Pt2 (trans Pt2 sc-de sc-para))
(setq Pt22 (getpoint "\nPonto correspondente a 2:"))
  (setq Pt22 (trans Pt22 sc-de sc-para))
(setq Pt3 (getpoint "\nPonto de Controlo 3:"))
  (setq Pt3 (trans Pt3 sc-de sc-para))
(setq Pt33 (getpoint "\nPonto correspondente a 3:"))
  (setq Pt33 (trans Pt33 sc-de sc-para))
(setq Pt4 (getpoint "\nPonto de Controlo 4:"))
  (setq Pt4 (trans Pt4 sc-de sc-para))
(setq Pt44 (getpoint "\nPonto correspondente a 4:"))
  (setq Pt44 (trans Pt44 sc-de sc-para))

(command "osmode" "0")

(parametros Pt1 Pt2 Pt3 Pt4 Pt11 Pt22 Pt33 Pt44)
(c:transf)
(print)
) 

;escolha de 4 linhas + 4 linhas e transformao

(defun c:4l4l1 ()
(print "")
  (setq l1 (entsel "Linha de Controlo 1 no paralela  linha 4:"))
(print "")
  (setq l2 (entsel "Linha de Controlo 2 no paralela  linha 1:"))
(print "")
  (setq l3 (entsel "Linha de Controlo 3 no paralela  linha 2:"))
(print "")
  (setq l4 (entsel "Linha de Controlo 4 no paralela  linha 3:"))
(print "")
  (setq l11 (entsel "Linha de Controlo correspondente a 1 no paralela  linha correspondente a 4:"))
(print "")
  (setq l22 (entsel "Linha de Controlo correspondente a 2 no paralela  linha correspondente a 1:"))
(print "")
  (setq l33 (entsel "Linha de Controlo correspondente a 3 no paralela  linha correspondente a 2:"))
(print "")
  (setq l44 (entsel "Linha de Controlo correspondente a 4 no paralela  linha correspondente a 3:"))

(command "osmode" "0")
  
  (pts_ctrl l1 l2)
  (setq Pt1 Pi_a.b)
  (pts_ctrl l2 l3)
  (setq Pt2 Pi_a.b)
  (pts_ctrl l3 l4)
  (setq Pt3 Pi_a.b)
  (pts_ctrl l4 l1)
  (setq Pt4 Pi_a.b)

  (pts_ctrl l11 l22)
  (setq Pt11 Pi_a.b)
  (pts_ctrl l22 l33)
  (setq Pt22 Pi_a.b)
  (pts_ctrl l33 l44)
  (setq Pt33 Pi_a.b)
  (pts_ctrl l44 l11)
  (setq Pt44 Pi_a.b)

(parametros Pt1 Pt2 Pt3 Pt4 Pt11 Pt22 Pt33 Pt44)
(c:transf)
(print)
)

;escolha de 1 linha + 1 linha (4xs) e transformao

(defun c:1l1l4 ()
(print "")
  (setq l1 (entsel "Linha de Controlo 1 no paralela  linha 4:"))
(print "")
  (setq l11 (entsel "Linha de Controlo correspondente a 1 no paralela  linha correspondente a 4:"))
(print "")
  (setq l2 (entsel "Linha de Controlo 2 no paralela  linha 1:"))
(print "")
  (setq l22 (entsel "Linha de Controlo correspondente a 2 no paralela  linha correspondente a 1:"))
(print "")
  (setq l3 (entsel "Linha de Controlo 3 no paralela  linha 2:"))
(print "")
    (setq l33 (entsel "Linha de Controlo correspondente a 3 no paralela  linha correspondente a 2:"))
(print "")
  (setq l4 (entsel "Linha de Controlo 4 no paralela  linha 3:"))
(print "")
  (setq l44 (entsel "Linha de Controlo correspondente a 4 no paralela  linha correspondente a 3:"))

(command "osmode" "0")
  
  (pts_ctrl l1 l2)
  (setq Pt1 Pi_a.b)
  (pts_ctrl l2 l3)
  (setq Pt2 Pi_a.b)
  (pts_ctrl l3 l4)
  (setq Pt3 Pi_a.b)
  (pts_ctrl l4 l1)
  (setq Pt4 Pi_a.b)

  (pts_ctrl l11 l22)
  (setq Pt11 Pi_a.b)
  (pts_ctrl l22 l33)
  (setq Pt22 Pi_a.b)
  (pts_ctrl l33 l44)
  (setq Pt33 Pi_a.b)
  (pts_ctrl l44 l11)
  (setq Pt44 Pi_a.b)

(parametros Pt1 Pt2 Pt3 Pt4 Pt11 Pt22 Pt33 Pt44)
(c:transf)
(print)
)


